@@ -4,6 +4,7 @@ import org.jmx4perl.JmxRequest;
 import org.jmx4perl.config.Restrictor;
 
 import javax.management.*;
+import java.io.IOException;
 import java.util.Set;
 
 /*
@@ -35,7 +36,7 @@ import java.util.Set;
  */
 public abstract class JsonRequestHandler {
 
-    // RestriOctor for restricting operations
+    // Restrictor for restricting operations
     protected final Restrictor restrictor;
 
     protected JsonRequestHandler(Restrictor pRestrictor) {
@@ -55,7 +56,7 @@ public abstract class JsonRequestHandler {
      * are done for you
      *
      * @return whether you want to have
-     * {@link #doHandleRequest(javax.management.MBeanServer, org.jmx4perl.JmxRequest)}
+     * {@link #doHandleRequest(javax.management.MBeanServerConnection, org.jmx4perl.JmxRequest)}
      * (<code>false</code>) or
      * {@link #doHandleRequest(java.util.Set, org.jmx4perl.JmxRequest)} (<code>true</code>) called.
      */
@@ -70,7 +71,7 @@ public abstract class JsonRequestHandler {
      * Does a check for restrictions as well
      *
      * @param server server to try
-     * @param request reqiest to process
+     * @param request request to process
      * @return the object result from the request
      *
      * @throws InstanceNotFoundException if the provided server cant handle the request
@@ -78,8 +79,8 @@ public abstract class JsonRequestHandler {
      * @throws ReflectionException
      * @throws MBeanException
      */
-    public Object handleRequest(MBeanServer server,JmxRequest request)
-            throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException {
+    public Object handleRequest(MBeanServerConnection server,JmxRequest request)
+            throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException, IOException {
         checkForType();
         return doHandleRequest(server,request);
     }
@@ -99,7 +100,7 @@ public abstract class JsonRequestHandler {
      * request.
      *
      * @param server server to try
-     * @param request reqiest to process
+     * @param request request to process
      * @return the object result from the request
      *
      * @throws InstanceNotFoundException
@@ -107,26 +108,26 @@ public abstract class JsonRequestHandler {
      * @throws ReflectionException
      * @throws MBeanException
      */
-    protected abstract Object doHandleRequest(MBeanServer server,JmxRequest request)
-            throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException;
+    protected abstract Object doHandleRequest(MBeanServerConnection server,JmxRequest request)
+            throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException, IOException;
 
     /**
      * Override this if you want to have all servers at once for processing the request
-     * (like need for merging infos as for a <code>list</code> command). This method
-     * is only called whem {@link #handleAllServersAtOnce()} returns <code>true</code>
+     * (like need for merging info as for a <code>list</code> command). This method
+     * is only called when {@link #handleAllServersAtOnce()} returns <code>true</code>
      *
      * @param servers all MBeans servers detected
      * @param request request to process
      * @return the object found
      */
-    public Object handleRequest(Set<MBeanServer> servers, JmxRequest request)
-            throws ReflectionException, InstanceNotFoundException, MBeanException, AttributeNotFoundException {
+    public Object handleRequest(Set<MBeanServerConnection> servers, JmxRequest request)
+            throws ReflectionException, InstanceNotFoundException, MBeanException, AttributeNotFoundException, IOException {
         checkForType();
         return doHandleRequest(servers,request);
     }
 
-    public Object doHandleRequest(Set<MBeanServer> servers, JmxRequest request)
-                throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException    {
+    public Object doHandleRequest(Set<MBeanServerConnection> servers, JmxRequest request)
+                throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException, IOException {
         return null;
     }
 }
