@@ -33,7 +33,7 @@ import java.io.Serializable;
  * @author roland
 * @since Jun 12, 2009
 */
-public class HistoryEntry implements Serializable {
+class HistoryEntry implements Serializable {
 
     private static final long serialVersionUID = 42L;
 
@@ -59,10 +59,6 @@ public class HistoryEntry implements Serializable {
     }
 
 
-    public int getMaxEntries() {
-        return maxEntries;
-    }
-
     public void setMaxEntries(int pMaxEntries) {
         maxEntries = pMaxEntries;
     }
