@@ -76,6 +76,12 @@ public class HistoryKey implements Serializable {
         if (pJmxRequest.getObjectNameAsString() == null) {
             throw new IllegalArgumentException("Mbean name must not be null");
         }
+        if (pJmxRequest.getObjectName().isPattern()) {
+            throw new IllegalArgumentException("Mbean name must not be a pattern");
+        }
+        if (pJmxRequest.getAttributeNames() != null && pJmxRequest.getAttributeNames().size() > 1) {
+            throw new IllegalArgumentException("A key cannot contain more than one attribute");
+        }
     }
 
     public HistoryKey(String pMBean, String pOperation, String pTarget) {
