@@ -1,5 +1,6 @@
 package org.jmx4perl.converter.json;
 
+import org.jmx4perl.JmxRequest;
 import org.jmx4perl.converter.StringToObjectConverter;
 import org.junit.After;
 import org.junit.Before;
@@ -47,7 +48,7 @@ public class ObjectToJsonConverterTest {
     @Before
     public void setup() {
         converter = new ObjectToJsonConverter(new StringToObjectConverter(),null);
-        converter.setupContext(0,0,0);
+        converter.setupContext(null,null,null,null);
     }
 
     @After
@@ -77,7 +78,7 @@ public class ObjectToJsonConverterTest {
         ctx.setMaxDepth(1);
         Map result = (Map) converter.extractObject(new SelfRefBean1(),new Stack<String>(),true);
         String c = (String) ((Map) result.get("bean2")).get("bean1");
-        assertTrue("Recurence detected",c.contains("Depth limit"));
+        assertTrue("Recurence detected",c.contains("bean1: toString"));
     }
 
     // ============================================================================
@@ -106,6 +107,10 @@ public class ObjectToJsonConverterTest {
         public boolean isStrong() {
             return strong;
         }
+
+        public String toString() {
+            return "bean1: toString";
+        }
     }
 
     class SelfRefBean2 {
