@@ -83,12 +83,12 @@ So, to fetch the C<"used"> value only, specify C<used> as path within the
 request. You can access deeper nested values by building up a path with "/" as
 separator. This looks a bit like a simplified form of XPath.
 
-=item max_depth, max_objects, max_list_size
+=item maxDepth, maxObjects, maxCollectionSize
 
 With these number you can restrict the size of the JSON structure
-returned. C<max_depth> gives the maximum nesting level of the JSON
-object,C<max_objects> returns the maximum number of objects to be returned in
-total and C<max_list_size> restrict the number of all arrays and collections
+returned. C<maxDepth> gives the maximum nesting level of the JSON
+object,C<maxObjects> returns the maximum number of objects to be returned in
+total and C<maxCollectionSize> restrict the number of all arrays and collections
 (maps, lists) in the answer. Note, that you should use this restrictions if you
 are doing massive bulk operations.
 
@@ -130,7 +130,7 @@ require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = (
            "READ","WRITE","EXEC","LIST", "SEARCH",
-           "REGNOTIF","REMNOTIF"
+           "REGNOTIF","REMNOTIF", "VERSION"
           );
 
 use constant READ => "read";
@@ -140,10 +140,11 @@ use constant LIST => "list";
 use constant SEARCH => "search";
 use constant REGNOTIF => "regnotif";
 use constant REMNOTIF => "remnotif";
+use constant VERSION => "version";
 
 my $TYPES = 
 { map { $_ => 1 } (READ, WRITE, EXEC, LIST, SEARCH,
-                   REGNOTIF, REMNOTIF) };
+                   REGNOTIF, REMNOTIF, VERSION) };
 
 =item  $req = new JMX::Jmx4Perl::Request(....);
 
@@ -158,10 +159,16 @@ parameter and a hashref containing named parameter for the request parameters
 (for the names, see above). Finally you can specify the arguments completely as
 a hashref, using 'type' for the entry specifying the request type.
 
-For the options C<max_depth>, C<max_objects> and C<max_list_size>, you can mix
+For the options C<maxDepth>, C<maxObjects> and C<maxCollectionSize>, you can mix
 them in into the hashref if using the hashed argument format. For the first
 format, these options are given as a final hashref.
 
+The option C<method> can be used to suggest a HTTP request method to use. By
+default, the agent decides automatically which HTTP method to use depending on
+the number of requests and whether an extended format should be used (which is
+only possible with an HTTP POST request). The value of this option can be
+either C<post> or C<get>, dependening on your preference. 
+
 If the request should be proxied through this request, a target configuration
 needs to be given as optional parameter. The target configuration consists of a
 JMX service C<url> and a optional environment, which is given as a key-value
@@ -185,6 +192,9 @@ named parameters are:
  Order    : $mbean, $attribute, $path
  Mandatory: $mbean, $attribute
 
+Note that C<$attribute> can be either a single name or a reference to a list
+of attribute names. 
+
 =item C<WRITE> 
 
  Order    : $mbean, $attribute, $value, $path
@@ -213,7 +223,6 @@ sub new {
     my $class = shift;
     my $type = shift;
     my $self;
-
     # Hash as argument
     if (ref($type) eq "HASH") {
         $self = $type;
@@ -229,16 +238,30 @@ sub new {
         } else {
             # Unnamed arguments
             $self = {type =>  $type};
+
+            # Options are given as last part
             my $opts = $_[scalar(@_)-1];
             if (ref($opts) eq "HASH") {
                 pop @_;
                 map { $self->{$_} = $opts->{$_} } keys %$opts;
-            } else {
-            }
+                if ($self->{method}) {
+                    # Canonicalize and verify
+                    &method($self,$self->{method});                    
+                }
+            } 
             if ($type eq READ) {
                 $self->{mbean} = shift;
                 $self->{attribute} = shift;
                 $self->{path} = shift;
+                # Use post for complex read requests
+                if (ref($self->{attribute}) eq "ARRAY") {
+                    if (&method($self) eq "GET") {
+                        # Was already explicitely set
+                        die "Cannot query for multiple attributes " . join(",",@{$self->{attributes}}) . " with a GET request"
+                          if ref($self->{attribute}) eq "ARRAY";
+                    }
+                    &method($self,"POST");
+                }
             } elsif ($type eq WRITE) {
                 $self->{mbean} = shift;
                 $self->{attribute} = shift;
@@ -252,7 +275,11 @@ sub new {
                 $self->{path} = shift;
             } elsif ($type eq SEARCH) {
                 $self->{mbean} = shift;
-            } else {
+                #No check here until now, is done on the server side as well.
+                #die "MBean name ",$self->{mbean}," is not a pattern" unless &is_mbean_pattern($self);
+            } elsif ($type eq VERSION) {
+                # No extra parameters required
+            }  else {
                 croak "Type ",$type," not supported yet";
             }
         }
@@ -262,6 +289,61 @@ sub new {
     return $self;
 }
 
+=item $req->method()
+
+=item $req->method("POST")
+
+Set the HTTP request method for this requst excplicitely. If not provided
+either during construction time (config key 'method') a prefered request
+method is determined dynamically based on the request contents.
+
+=cut 
+
+sub method {
+    my $self = shift;
+    my $value = shift;
+    if (defined($value)) {
+        die "Unknown request method ",$value if length($value) && $value !~ /^(POST|GET)$/i;
+        $self->{method} = uc($value);
+    }
+    return defined($self->{method}) ? $self->{method} : "";
+}
+
+=item $req->is_mbean_pattern
+
+Returns true, if the MBean name used in this request is a MBean pattern (which
+can be used in C<SEARCH> or for C<READ>) or not
+
+=cut
+
+sub is_mbean_pattern {
+    my $self = shift;
+    my $mbean = shift || $self->{mbean};
+    return 1 unless $mbean;
+    my ($domain,$rest) = split(/:/,$mbean,2);
+    return 1 if $domain =~ /[*?]/;
+    return 1 if $rest =~  /\*$/;
+
+    while ($rest) {
+        #print "R: $rest\n";
+        $rest =~ s/([^=]+)\s*=\s*//;
+        my $key = $1;
+        my $value;
+        if ($rest =~ /^"/) {
+            $rest =~ s/("(\\"|[^"])+")(\s*,\s*|$)//;
+            $value = $1;
+            # Pattern in quoted values must not be preceded by a \
+            return 1 if $value =~ /(?<!\\)[\*\?]/;
+        } else {
+            $rest =~ s/([^,]+)(\s*,\s*|$)//;
+            $value = $1;
+            return 1 if $value =~ /[\*\?]/;
+        }
+        #print "K: $key V: $value\n";
+    }
+    return 0;
+}
+
 =item $request->get("type")
 
 Get a request parameter
@@ -296,9 +378,14 @@ sub TO_JSON {
     my $ret = {
                type => $self->{type} ? uc($self->{type}) : undef,
               };
-    for my $k (qw(mbean attribute path value operation arguments max_depth max_objects max_list_size target)) {
-        $ret->{$k} = $self->{$k} if $self->{$k};
+    for my $k (qw(mbean attribute path value operation arguments target)) {
+        $ret->{$k} = $self->{$k} if defined($self->{$k});
     }
+    my %config;
+    for my $k (qw(maxDepth maxObjects maxCollectionSize ignoreErrors)) {
+        $config{$k} = $self->{$k} if defined($self->{$k});
+    }
+    $ret->{config} = \%config if keys(%config);
     return $ret;
 }
 
